% This function computes the phase difference between two wide band
% signals. It selects the same harmonic frequency in each signals frequency
% space and determines the phase difference between these harmonics.
function [PhDiff,fftDiff] = funPhaseDiff(x, y)
% function to compute the phase difference between two signals

% INPUTS 
% x,y - vectors in time domain (can be complex valued)

% OUTPUT
% PhDiff - phase difference (given in radians)

% represent the signals as column-vectors
x = x(:);
y = y(:);

% remove the DC component of the signals
x = x - mean(x);
y = y - mean(y);

% signals length calculation
xlen = length(x);
ylen = length(y);

% windows generation
xwin = hanning(xlen, 'periodic');
ywin = hanning(ylen, 'periodic');

% perform fft on the signals
X = fft(x.*xwin); 
Y = fft(y.*ywin);

fftDiff = sum(abs(X)-abs(Y));

% fundamental frequency detection
[~, indx] = max(abs(X));
[~, indy] = max(abs(Y));

% phase difference estimation
PhDiff = angle(Y(indx)) - angle(X(indx));
% PhDiff = max(angle(Y) - angle(X));      %select positive frequency shift

end