function [LocMat,LocMatLamb] = funRENEWAntLoc(numAnts)
%   Function to give the locations of the Rx antenna elements on the RENEW
%   base station array

%   Inputs
%       ants is a vector [1,...,numRx]

%   Outputs
%       LocMat is a matrix of antenna element locations 3 x numRx



% element spacing in meters
dX = 39.4e-3;   % approx \lambda/2
dY = 61.0e-3;

elementsPerRow = 8; 
numRows = numAnts/(2*elementsPerRow);


holdMat = zeros(3,numRows*elementsPerRow);
LocMat = zeros(3,numAnts);
LocMatLamb = zeros(3,numAnts);

for rowInd = 1:numRows
    holdMat( 1 , elementsPerRow*(rowInd-1)+1 : elementsPerRow*rowInd ) = [0:elementsPerRow-1];
    holdMat( 2 , elementsPerRow*(rowInd-1)+1 : elementsPerRow*rowInd ) = (rowInd-1);%*(dY/dX) ;
end


for eleInd = 1:numRows*elementsPerRow
    LocMatLamb(:,2*eleInd-1) = holdMat(:,eleInd);
    LocMatLamb(:,2*eleInd) = holdMat(:,eleInd);
end

LocMat = LocMatLamb.*[dX; dY; 0];

end