%   function [V] = mimoStuff_ant(filename)


function [V] = mimoStuff_ant(filename, antLocREF, sdr_Map)
%function [dataCube, f0, antenna] = mimoStuff_ant(filename, antLocREF, sdr_Map)


            usePhaseCal = true;        % set true to apply DFT-ML phase correction
            useREFcorrect = true;       % set true to account for phase offset from TX reference not on boresight
            %startFrame = 1;
            startFrame = 600;
            numFrame = 300;
            %numFrame = 2000;
            %% Set up some PHY stuff
            c = physconst('LightSpeed');
            f0 = h5readatt(filename,'/Data','FREQ');
            lambda = c/f0;
            numRxAnts = double(h5readatt(filename,'/Data','ANT_TOTAL'));
            dX = 39.4e-3;   % approx \lambda/2
            dY = 61.0e-3;   % taken from West's lens paper

            %% Setting up the RX array locations
            % first get all the BS antennas (rx) locations (relative)
            [antLoc,antLocLamb] = funRENEWAntLoc(numRxAnts);
            % adjust spacing so that center of array is the origin
            antLoc = antLoc + repmat([-0.137900000000000;-0.0915000000000000;0],1,length(antLoc));

            %% Correct for Phase offset from TX Reference off Boresight
            %--------------------------------------------------------------------------
            antenna = phased.IsotropicAntennaElement(...
                'FrequencyRange',[1.6e9 5.6e9]); %use with 5 GHz carrier [2e9 8e9] 
            % Create conformal antenna array object
            normal_az = ones(1,length(antLoc)).*(0);             %azimuth in degrees
            normal_el = ones(1,length(antLoc)).*(90-0);          %elevation in degrees, note matlab bases theta off of x-y plane
            arrayRX = phased.ConformalArray('Element',antenna,...
                'ElementPosition',antLoc,'ElementNormal',[normal_az;normal_el]);
            viewArray(arrayRX,'ShowNormals',true)
            phaseCentRX = [0;0;0];

            if(useREFcorrect == true)     %execute if not at boresight
                % get angles between reference TX and the receive array
                [~,REF_rx_angles_corr] = rangeangle(antLocREF,phaseCentRX);

                % create time delay object corresponding to the array
                delayObjectREF = phased.ElementDelay('SensorArray',arrayRX);

                % calculate the time delays
                timeDelaysREF = delayObjectREF([REF_rx_angles_corr]); % size [M,N], given in seconds
                phaseDelaysREF = exp( -1j * 2 * pi * f0 * timeDelaysREF );
            end

            %% Import and format reference node pilotCube
            %--------------------------------------------------------------------------
            %start = [   1  1 1 1 startFrame];      % index to START at
            start = [1 1 3 1 startFrame];      % index to START at
            %count = [2240 36 1 1 numFrame];      % size of the thing to collect
            antenna_number = h5readatt(filename,'/Data','ANT_NUM');
            antenna_number = double(antenna_number); 
            count = [2240 antenna_number 1 1 numFrame];      % size of the thing to collect
            stride = [  1  1 1 1   1];      % interval between data
            refUser_pilots = h5read(filename,'/Data/Pilot_Samples',start,count,stride);
            refUser_pilots = squeeze(refUser_pilots);  % size [2*numIQsamps numRx numFrames]

            % the symbols in the first dimension are I and Q samples
            %   make data into complex IQ samples along first dimension
            refUser_pilots = double(refUser_pilots);
            scaleFactor = 2^(-15);
            refUser_pilots_complex = scaleFactor * ( refUser_pilots(1:2:end,:,:) + 1i.*refUser_pilots(2:2:end,:,:) ); % size [numIQsamps numRx numFrames]

            % stack all the frames onto each other 
            pilotCube = zeros(size(refUser_pilots_complex,1)*size(refUser_pilots_complex,3), size(refUser_pilots_complex,2) );
            for rxInd = 1:size(refUser_pilots_complex,2)
                holder = zeros(size(refUser_pilots_complex,1),size(refUser_pilots_complex,3));
                holder(:,:) = squeeze(refUser_pilots_complex(:,rxInd,:));
                pilotCube(:,rxInd) = holder(:);
            end
            % size [numIQsamps*numFrames numRx]
            pilotLength = size(pilotCube,1);

            %% Correct for Reference Node off of Boresight
            if ( useREFcorrect == true)
                pilotCube = pilotCube.*(phaseDelaysREF).';
            end

            %% Perform phase calibration on reference user pilot data
            %--------------------------------------------------------------------------
            if (usePhaseCal == true)
                % preallocate
                phaseDiffVec = zeros(1,size(pilotCube,2));
                phaseDiffVec_CHECK = zeros(1,size(pilotCube,2));
                pilotCube_phCorr = zeros(size(pilotCube));

                for rxInd = 1:size(pilotCube,2)
                    % calculate the phase difference between antenna 1 and each other antenna
                    phaseDiffVec(rxInd) = funPhaseDiff( pilotCube(:,1) , pilotCube(:,rxInd) );

                    % apply phase correction to each antenna
                    pilotCube_phCorr(:,rxInd) = pilotCube(:,rxInd) .* exp(-1j * phaseDiffVec(rxInd));

                    % sanity check - these should all be close to zero now
                    phaseDiffVec_CHECK(rxInd) = funPhaseDiff( pilotCube_phCorr(:,1) , pilotCube_phCorr(:,rxInd) );

                end    
                % NOTES
                % If we do VCZ on pilotCube_phCorr, we should see our reference user at the
                % (0,0) (azimuth,elevation)
            end

            %% Import and format client node dataCube
            %--------------------------------------------------------------------------
            %start = [1 1 3 1 startFrame];      % index to START at
            start = [   1  1 1 1 startFrame];      % index to START at
            %count = [2240 36 1 1 numFrame];      % size of the thing to collect
            antenna_number = h5readatt(filename,'/Data','ANT_NUM');
            antenna_number = double(antenna_number); 
            count = [2240 antenna_number 1 1 numFrame];
            stride = [1 1 1 1 1];      % interval between data
            %clientUser_data = h5read(filename,'/Data/UplinkData',start,count,stride);
            clientUser_data = h5read(filename,'/Data/Pilot_Samples',start,count,stride); % changed which dataset it being used
            clientUser_data = squeeze(clientUser_data);  % size [2*numIQsamps numRx numFrames]

            % the symbols in the first dimension are I and Q samples
            %   make data into complex IQ samples along first dimension
            clientUser_data = double(clientUser_data);
            scaleFactor = 2^(-15);
            clientUser_data_complex = scaleFactor * ( clientUser_data(1:2:end,:,:) + 1i.*clientUser_data(2:2:end,:,:) ); % size [numIQsamps numRx numFrames]

            % stack all the frames onto each other 
            dataCube = zeros(size(clientUser_data_complex,1)*size(clientUser_data_complex,3), size(clientUser_data_complex,2) );
            for rxInd = 1:size(clientUser_data_complex,2)
                holder = zeros(size(clientUser_data_complex,1),size(clientUser_data_complex,3));
                holder(:,:) = squeeze(clientUser_data_complex(:,rxInd,:));
                dataCube(:,rxInd) = holder(:);
            end
            % size [numIQsamps*numFrames numRx]
            signalLength = size(dataCube,1);

            %figure; plot(1:length(dataCube),real(dataCube(:,20))); hold on; plot(1:length(dataCube),imag(dataCube(:,20))); % spot check

            % apply the phase correction from the last section 
            if (usePhaseCal == true)
                dataCube = dataCube .* exp( -1j * phaseDiffVec );
            %     dataCube = pilotCube_phCorr;
            % dataCube = pilotCube;
            end

            % EDIT - ONLY USE ANTENNAS WITH SAME POLARIZATION
            numRxAnts = numRxAnts/2;
            
            %dataCube = dataCube(:,1:2:end);
            % Even
            dataCube = dataCube(:,2:2:end);
            
            pilotCube = pilotCube(:,1:2:end);
            antLoc = antLoc(:,1:2:end);
            
            orgin_dataCube = dataCube;
            
            %% Zeroing done by INTERN TYLOR

sdr_Data = h5readatt(filename,'/Data','BS_SDR_ID');
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% TY'S INDEX FINDER
       size_map = size(sdr_Map);
       size_Data = size(sdr_Data);
       sdr_Data_zeroed(1:size_map)="";
       for k2=1:size_Data
           sdr_Data_zeroed(k2)=sdr_Data(k2);
       end
       %disp(sdr_Data_zeroed)
       for k1=1:size_map
           if ismember(sdr_Map(k1),sdr_Data_zeroed)
           elseif k1==1 && ~ismember(sdr_Map(k1),sdr_Data_zeroed(k1))
               sdr_Data_zeroed=["miss" sdr_Data_zeroed(k1:size_map)];
           else
               sdr_Data_zeroed = [sdr_Data_zeroed(1:k1-1) "miss" sdr_Data_zeroed(k1:size_map)];
           end
       end
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       % THIS ARRAY HOLDS INDEX LOCATIONS OF MISSING ANT-Ty
       numANTmiss = size_map - size_Data;
       misANTloc(1:numANTmiss) = "";
       for k4=1:numANTmiss
           for k3 = 1 : size_map
              if sdr_Data_zeroed(k3) == "miss"
                  comparitor = num2str(k3);
                  if ismember(comparitor,misANTloc)
                       
                  else
                      misANTloc(k4)= k3;
                  end
              end
           end
       end
       count = 0;
       for v =1:1:size_map
           point = num2str(v);
           if ismember(point,misANTloc)
               Zeroing(1:1120*numFrame,1) = 0+0i;
               dataCube = [dataCube(:,1:v-1) Zeroing(:,1) dataCube(:,v:size_Data+count) ];
               count=count+1;
           end
       end
      
            
            %%
            z = [2:8];
            V_2 = [2:8];
            for a = 1:7
            fc = f0;
            fprintf('Ant for array fun: %d\n', V_2(a));
            lambda = physconst('LightSpeed')/fc;
            %antenna = phased.IsotropicAntennaElement(...'FrequencyRange',[1.6e9 5.6e9]); %use with 5 GHz carrier [2e9 8e9] %Already given
            array = phased.ULA('Element',antenna,'NumElements',V_2(a),'ElementSpacing',lambda/2);
            
            estimator = phased.BeamscanEstimator('SensorArray',array,...
                'OperatingFrequency',fc,'ScanAngles',-90:90,...
                'DOAOutputPort',true,'NumSignals',1);
            
            %here
            %%
            % ------ Edited Here ------- 
            % Here is where we can change the number of antennas being used
            
                fprintf('Ant number: %d\n', z(a));
                rxsig = dataCube(:,1:z(a));
                %rxsig = dataCube(:,1:8);
                %rxsig = dataCube;
                %%%
                [y,sigang] = estimator(rxsig);
                disp('Values')
                disp(sigang)
                V(a) = sigang;
                % plotSpectrum(estimator)

            end
            
end